package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlConceptCommon;
import com.diagbot.entity.KlDisease;
import com.diagbot.entity.KlLibraryInfo;
import com.diagbot.entity.KlRelation;
import com.diagbot.entity.KlRule;
import com.diagbot.entity.KlRuleBase;
import com.diagbot.entity.KlRuleCondition;
import com.diagbot.entity.node.ExportNode;
import com.diagbot.entity.node.ExportRelation;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.repository.ExportNodeRepository;
import com.diagbot.repository.ExportRelationRepository;
import com.diagbot.service.KlConceptCommonService;
import com.diagbot.service.KlConceptService;
import com.diagbot.service.KlDiseaseService;
import com.diagbot.service.KlLibraryInfoService;
import com.diagbot.service.KlRelationService;
import com.diagbot.service.KlRuleBaseService;
import com.diagbot.service.KlRuleConditionService;
import com.diagbot.service.KlRuleService;
import com.diagbot.util.Cn2SpellUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ImportDataVO;
import com.diagbot.vo.KlRuleAnalyVO;
import com.diagbot.vo.KlRuleConditionVO;
import com.diagbot.vo.TestAllVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @author: gaodm
 * @time: 2021/1/21 9:34
 */
@Component
@Slf4j
public class ExportFacade {
    @Autowired
    private ExportNodeRepository exportNodeRepository;
    @Autowired
    private ExportRelationRepository exportRelationRepository;
    @Autowired
    @Qualifier("klConceptServiceImpl")
    private KlConceptService klConceptService;
    @Autowired
    @Qualifier("klLibraryInfoServiceImpl")
    private KlLibraryInfoService klLibraryInfoService;
    @Autowired
    @Qualifier("klConceptCommonServiceImpl")
    private KlConceptCommonService klConceptCommonService;
    @Autowired
    @Qualifier("klDiseaseServiceImpl")
    private KlDiseaseService klDiseaseService;
    @Autowired
    @Qualifier("klRelationServiceImpl")
    private KlRelationService klRelationService;
    @Autowired
    private TestFacade testFacade;
    @Autowired
    @Qualifier("klRuleServiceImpl")
    private KlRuleService klRuleService;
    @Autowired
    @Qualifier("klRuleBaseServiceImpl")
    private KlRuleBaseService klRuleBaseService;
    @Autowired
    @Qualifier("klRuleConditionServiceImpl")
    private KlRuleConditionService klRuleConditionService;

    public final static List<String> operatorList = new ArrayList<String>() {
        {
            add(">");
            add("＞");
            add("≥");
            add(">=");
            add("＞=");
            add("=");
            add("＜");
            add("<");
            add("<=");
            add("＜=");
            add("≤");
        }
    };
    public final static List<String> operator_eq = new ArrayList<String>() {
        {
            add("=");
        }
    };
    public final static List<String> operator_gt = new ArrayList<String>() {
        {
            add(">");
            add("＞");
        }
    };
    public final static List<String> operator_ge = new ArrayList<String>() {
        {
            add("≥");
            add(">=");
            add("＞=");
        }
    };
    public final static List<String> operator_lt = new ArrayList<String>() {
        {
            add("＜");
            add("<");
        }
    };
    public final static List<String> operator_le = new ArrayList<String>() {
        {
            add("<=");
            add("＜=");
            add("≤");
        }
    };


    public Boolean gdbExportNode() {
        //图谱数据节点处理
        graphNodeDeal();
        return true;
    }

    private void graphNodeDeal() {
        //疾病处理
        List<ExportNode> nodes = exportNodeRepository.getDisease();
        import2MySql(nodes, 100);
        //药品通用名
        nodes.clear();
        nodes = exportNodeRepository.getDrug();
        import2MySql(nodes, 101);
        //药品剂型
        nodes.clear();
        nodes = exportNodeRepository.getDrugForm();
        import2MySql(nodes, 102);
        //症状
        nodes.clear();
        nodes = exportNodeRepository.getSymptom();
        import2MySql(nodes, 103);
        //体格检查结果
        nodes.clear();
        nodes = exportNodeRepository.getVital();
        import2MySql(nodes, 105);
        //手术和操作
        nodes.clear();
        nodes = exportNodeRepository.getOeration();
        import2MySql(nodes, 106);
        //实验室检查套餐
        nodes.clear();
        nodes = exportNodeRepository.getLisPackage();
        import2MySql(nodes, 107);
        //实验室检查子项目
        nodes.clear();
        nodes = exportNodeRepository.getLisItem();
        import2MySql(nodes, 108);
        //辅助检查项目
        nodes.clear();
        nodes = exportNodeRepository.getPacs();
        import2MySql(nodes, 109);
        //辅助检查子项目
        nodes.clear();
        nodes = exportNodeRepository.getPacsSubItem();
        import2MySql(nodes, 110);
        //辅助检查描述
        nodes.clear();
        nodes = exportNodeRepository.getPacsDesc();
        import2MySql(nodes, 111);
        //辅助检查结果
        nodes.clear();
        nodes = exportNodeRepository.getPacsResult();
        import2MySql(nodes, 112);
        //输血类型
        nodes.clear();
        nodes = exportNodeRepository.getTransfusion();
        import2MySql(nodes, 113);
        //科室
        nodes.clear();
        nodes = exportNodeRepository.getDept();
        import2MySql(nodes, 115);
        //性别
        nodes.clear();
        nodes = exportNodeRepository.getSex();
        import2MySql(nodes, 116);
        //其他过敏原
        nodes.clear();
        nodes = exportNodeRepository.getAllergen();
        import2MySql(nodes, 119);
        //医疗器械及物品
        nodes.clear();
        nodes = exportNodeRepository.getApparatus();
        import2MySql(nodes, 120);
        //药品化学物质类别
        nodes.clear();
        nodes = exportNodeRepository.getDrugTypeA();
        import2MySql(nodes, 301);
        //药品治疗学类别
        nodes.clear();
        nodes = exportNodeRepository.getDrugTypeB();
        import2MySql(nodes, 302);
        //药品药理学类别
        nodes.clear();
        nodes = exportNodeRepository.getDrugTypeC();
        import2MySql(nodes, 303);
        //药品解剖学类别
        nodes.clear();
        nodes = exportNodeRepository.getDrugTypeD();
        import2MySql(nodes, 304);
    }

    private void import2MySql(List<ExportNode> nodes, Integer type) {
        List<KlLibraryInfo> libraryInfos = new ArrayList<>();
        List<String> names = new ArrayList<>();
        klLibraryInfoService.remove(new QueryWrapper<KlLibraryInfo>().eq("type_id", type));
        klConceptService.remove(new QueryWrapper<KlConcept>().eq("lib_type", type));
        for (ExportNode exportNode : nodes) {
            if (StringUtil.isNotBlank(exportNode.getName())
                    && !names.contains(exportNode.getName())) {
                KlLibraryInfo klLibraryInfo = new KlLibraryInfo();
                klLibraryInfo.setName(exportNode.getName());
                klLibraryInfo.setIsConcept(1);
                klLibraryInfo.setTypeId(type);
                klLibraryInfo.setSpell(Cn2SpellUtil.converterToFirstSpell(exportNode.getName()));
                libraryInfos.add(klLibraryInfo);
                names.add(exportNode.getName());
            }

        }
        klLibraryInfoService.saveBatch(libraryInfos);
        List<KlConcept> klConcepts = new ArrayList<>();
        for (KlLibraryInfo klLibraryInfo : libraryInfos) {
            KlConcept klConcept = new KlConcept();
            klConcept.setLibId(klLibraryInfo.getId());
            klConcept.setLibName(klLibraryInfo.getName());
            klConcept.setLibType(type);
            klConcepts.add(klConcept);
        }
        klConceptService.saveBatch(klConcepts);
        Map<Long, KlConcept> map = EntityUtil.makeEntityMap(klConcepts, "libId");
        for (KlLibraryInfo klLibraryInfo : libraryInfos) {
            klLibraryInfo.setConceptId(map.get(klLibraryInfo.getId()).getId());
        }
        klLibraryInfoService.updateBatchById(libraryInfos);
        if (type.equals(100)) {
            klConceptCommonService.remove(new QueryWrapper<KlConceptCommon>().eq("is_deleted", IsDeleteEnum.N.getKey()));
            klDiseaseService.remove(new QueryWrapper<KlDisease>().eq("is_deleted", IsDeleteEnum.N.getKey()));
            Map<String, ExportNode> nodeMap = EntityUtil.makeEntityMap(nodes, "name");
            List<KlConceptCommon> klConceptCommons = new ArrayList<>();
            List<KlDisease> klDiseases = new ArrayList<>();
            for (KlConcept klConcept : klConcepts) {
                KlConceptCommon klConceptCommon = new KlConceptCommon();
                klConceptCommon.setConceptId(klConcept.getId());
                KlDisease klDisease = new KlDisease();
                klDisease.setConceptId(klConcept.getId());
                ExportNode exportNode = nodeMap.get(klConcept.getLibName());
                if (null != exportNode) {
                    klConceptCommon.setSexType(Integer.valueOf(exportNode.getSex()));
                    String[] age = exportNode.getAgeRange().split("-");
                    klConceptCommon.setMinAge(Double.valueOf(age[0]));
                    if (age.length == 1) {
                        klConceptCommon.setMaxAge(200.0d);
                    } else {
                        klConceptCommon.setMaxAge(Double.valueOf(age[1]));
                    }
                    klDisease.setIcd10Code(exportNode.getIcd10Code());
                    klConceptCommons.add(klConceptCommon);
                    klDiseases.add(klDisease);
                }
            }
            klConceptCommonService.saveBatch(klConceptCommons);
            klDiseaseService.saveBatch(klDiseases);
        }
    }

    public Boolean gdbExportRelation() {
        //图谱关系处理
        graphRelationDeal();
        return true;
    }

    private void graphRelationDeal() {
        List<KlConcept> klConcepts
                = klConceptService.list(new QueryWrapper<KlConcept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
        );
        Map<String, KlConcept> map
                = EntityUtil.makeEntityMapByKeys(klConcepts, "_", "libName", "libType");
        //药品化学物质类别树形结构
        List<ExportRelation> exportRelations = exportRelationRepository.getDrugRelationA();
        import2MySqlR(exportRelations, 301, 101, map);
        //药品治疗学类别树形结构
        exportRelations.clear();
        exportRelations = exportRelationRepository.getDrugRelationB();
        import2MySqlR(exportRelations, 302, 101, map);
        //药品药理学类别树形结构
        exportRelations.clear();
        exportRelations = exportRelationRepository.getDrugRelationC();
        import2MySqlR(exportRelations, 303, 101, map);
        //药品解剖学类别树形结构
        exportRelations.clear();
        exportRelations = exportRelationRepository.getDrugRelationD();
        import2MySqlR(exportRelations, 304, 101, map);
        //实验室检查类别树形结构
        exportRelations.clear();
        exportRelations = exportRelationRepository.getLisRelation();
        import2MySqlR(exportRelations, 107, 108, map);
        //辅助检查类别树形结构
        exportRelations.clear();
        exportRelations = exportRelationRepository.getPacsRelation();
        import2MySqlR(exportRelations, 109, 110, map);
    }

    private void import2MySqlR(List<ExportRelation> exportRelations, Integer stype, Integer etype, Map<String, KlConcept> map) {
        import2MySqlR(exportRelations, stype, etype, 600, map);
    }

    private void import2MySqlR(List<ExportRelation> exportRelations, Integer stype, Integer etype, Integer rType, Map<String, KlConcept> map) {
        List<KlRelation> klRelations = new ArrayList<>();
        for (ExportRelation exportRelation : exportRelations) {
            if (null != map.get(exportRelation.getSname() + "_" + stype)
                    && null != map.get(exportRelation.getEname() + "_" + etype)) {
                KlRelation klRelation = new KlRelation();
                klRelation.setStartId(map.get(exportRelation.getSname() + "_" + stype).getId());
                klRelation.setEndId(map.get(exportRelation.getEname() + "_" + etype).getId());
                klRelation.setRelationId(rType);
                klRelations.add(klRelation);
            }
        }
        klRelationService.saveBatch(klRelations);
    }

    public Map<String, Object> importExcelGraphDB() {
        graphRelationDealdiagnose();
        return null;
    }

    private void graphRelationDealdiagnose() {
        List<KlConcept> klConcepts
                = klConceptService.list(new QueryWrapper<KlConcept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
        );
        Map<String, KlConcept> map
                = EntityUtil.makeEntityMapByKeys(klConcepts, "_", "libName", "libType");
        //主症状
        List<ExportRelation> exportRelations = exportRelationRepository.symptomMainPush();
        import2MySqlR(exportRelations, 100, 103, 501, map);
        //次症状
        exportRelations.clear();
        exportRelations = exportRelationRepository.symptomSecPush();
        import2MySqlR(exportRelations, 100, 103, 502, map);
        //体格检查结果
        exportRelations.clear();
        exportRelations = exportRelationRepository.vitalPush();
        import2MySqlR(exportRelations, 100, 105, 503, map);
        //实验室检查套餐
        exportRelations.clear();
        exportRelations = exportRelationRepository.lisPush();
        import2MySqlR(exportRelations, 100, 107, 504, map);
        //辅助检查项目
        exportRelations.clear();
        exportRelations = exportRelationRepository.pacsPush();
        import2MySqlR(exportRelations, 100, 109, 505, map);
        //药品通用名
        exportRelations.clear();
        exportRelations = exportRelationRepository.drugPush();
        import2MySqlR(exportRelations, 100, 101, 506, map);
        //手术和操作
        exportRelations.clear();
        exportRelations = exportRelationRepository.operationPush();
        import2MySqlR(exportRelations, 100, 106, 507, map);
        //鉴别诊断
        exportRelations.clear();
        exportRelations = exportRelationRepository.diffDiagnosePush();
        import2MySqlR(exportRelations, 100, 100, 508, map);
    }

    public Map<String, Object> importExcel(MultipartFile file) {
        List<ImportDataVO> data = ExcelUtils.importExcel(file, 0, 1, ImportDataVO.class);
        List<KlConcept> klConcepts
                = klConceptService.list(new QueryWrapper<KlConcept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
        );
        Map<String, KlConcept> map
                = EntityUtil.makeEntityMapByKeys(klConcepts, "_", "libName", "libType");
        List<KlRelation> klRelations = new ArrayList<>();
        List<String> relations = new ArrayList<>();
        for (ImportDataVO importDataVO : data) {
            //主症状
            setKlRelations(importDataVO.getDiseasePush(), importDataVO.getSymptomMainPush(), 100, 103, 501, map, klRelations, relations);
            //次症状
            setKlRelations(importDataVO.getDiseasePush(), importDataVO.getSymptomSecPush(), 100, 103, 502, map, klRelations, relations);
            //体格检查结果
            setKlRelations(importDataVO.getDiseasePush(), importDataVO.getVitalPush(), 100, 105, 503, map, klRelations, relations);
            //实验室检查套餐
            setKlRelations(importDataVO.getDiseasePush(), importDataVO.getLisPush(), 100, 107, 504, map, klRelations, relations);
            //辅助检查项目
            setKlRelations(importDataVO.getDiseasePush(), importDataVO.getPacsPush(), 100, 109, 505, map, klRelations, relations);
            //辅助检查子项目
            setKlRelations(importDataVO.getDiseasePush(), importDataVO.getPacsPush(), 100, 110, 505, map, klRelations, relations);
            //药品通用名
            setKlRelations(importDataVO.getDiseasePush(), importDataVO.getDrugPush(), 100, 101, 506, map, klRelations, relations);
            //手术和操作
            setKlRelations(importDataVO.getDiseasePush(), importDataVO.getOperationPush(), 100, 106, 507, map, klRelations, relations);
        }
        klRelationService.saveBatch(klRelations);
        return null;
    }

    private void setKlRelations(String startStr, String endStr, Integer stype, Integer etype, Integer rtype,
                                Map<String, KlConcept> map, List<KlRelation> klRelations, List<String> relations) {
        if (StringUtil.isNotBlank(startStr)
                && StringUtil.isNotBlank(endStr)) {
            List<String> ends = Arrays.asList(endStr.replaceAll("；", ";").split(";"));
            for (String s : ends) {
                if (null != map.get(startStr + "_" + stype)
                        && null != map.get(s + "_" + etype)) {
                    KlRelation klRelation = new KlRelation();
                    klRelation.setStartId(map.get(startStr.trim() + "_" + stype).getId());
                    klRelation.setEndId(map.get(s.trim() + "_" + etype).getId());
                    klRelation.setRelationId(rtype);
                    String r = map.get(startStr.trim() + "_" + stype).getId() + "_" + rtype + "_" + map.get(s.trim() + "_" + etype).getId();
                    if (!relations.contains(r)) {
                        relations.add(r);
                        klRelations.add(klRelation);
                    }

                }
            }
        }
    }

    public Map<String, Object> importExcelTip(TestAllVO testAllVO) {
        if (StringUtil.isBlank(testAllVO.getPath())) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "参数为空，未填写【文件所在路径】");
        }
        String path = testAllVO.getPath();
        Map<String, Object> paramMap = new LinkedHashMap<>();
        Map<String, String> mapPath = new LinkedHashMap<>();
        mapPath.put("【危急值_化验】", path + File.separator + "【危急值_化验】.xlsx");
        mapPath.put("【危急值_辅检】", path + File.separator + "【危急值_辅检】.xlsx");
        mapPath.put("【其他值提醒_化验】", path + File.separator + "【其他值提醒_化验】.xlsx");
        mapPath.put("【其他值提醒_辅检】", path + File.separator + "【其他值提醒_辅检】.xlsx");
        mapPath.put("【其他值提醒_输血】", path + File.separator + "【其他值提醒_输血】.xlsx");
        mapPath.put("【高危药品】", path + File.separator + "【高危药品】.xlsx");
        mapPath.put("【高危手术整合】", path + File.separator + "【高危手术整合】.xlsx");
        mapPath.put("【开单合理性】", path + File.separator + "【开单合理性】.xlsx");
        mapPath.put("【开单合理性_输血】", path + File.separator + "【开单合理性_输血】.xlsx");
        paramMap.put("mapPath", mapPath);

        List<KlConcept> klConcepts
                = klConceptService.list(new QueryWrapper<KlConcept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
        );
        Map<String, KlConcept> map
                = EntityUtil.makeEntityMapByKeys(klConcepts, "_", "libName", "libType");
        //高危药品
        //drugHighRisk(mapPath, map);
        //高危手术
        //optHighRisk(mapPath, map);
        //危急值化验
        lisCriticalValues(mapPath, map);
        //危急值辅检
        //pacsCriticalValues(mapPath, map);
        return null;
    }

    private void lisCriticalValues(Map<String, String> mapPath, Map<String, KlConcept> map) {
        MultipartFile file = null;
        file = testFacade.getMulFileByPath(mapPath.get("【危急值_化验】"));
        List<ImportDataVO> data = ExcelUtils.importExcel(file, 0, 1, ImportDataVO.class);
        List<String> lis = new ArrayList<>();
        for (ImportDataVO importDataVO : data) {
            if (StringUtil.isNotBlank(importDataVO.getLisGBName())) {
                String lisStr = importDataVO.getLisGBName().trim();
                if (StringUtil.isNotBlank(importDataVO.getCriticalAge())) {
                    lisStr = lisStr + "_" + importDataVO.getCriticalAge().trim();
                }
                if (!lis.contains(lisStr)
                        && (null != map.get(importDataVO.getLisGBName().trim() + "_108"))) {
                    lis.add(lisStr);
                    KlRule klRule = new KlRule();
                    klRule.setConceptId(map.get(importDataVO.getLisGBName().trim() + "_108").getId());
                    klRule.setDescription("危急值_化验_" + lisStr);
                    klRule.setRuleType(3);
                    klRule.setHasSubCond(1);
                    klRuleService.save(klRule);
                    KlRuleBase klRuleBase = new KlRuleBase();
                    klRuleBase.setConceptId(map.get(importDataVO.getLisGBName().trim() + "_108").getId());
                    klRuleBase.setDescription("比较条件:" + importDataVO.getLisGBName().trim());
                    if (null != importDataVO.getCriticalMin()) {
                        klRuleBase.setType(2);
                        klRuleBase.setMinOperator("<=");
                        klRuleBase.setMinValue(importDataVO.getCriticalMin().toString());
                        if (StringUtil.isNotBlank(importDataVO.getUnit())) {
                            klRuleBase.setMinUnit(importDataVO.getUnit().trim());
                        }
                    }
                    if (null != importDataVO.getCriticalMax()) {
                        klRuleBase.setType(2);
                        klRuleBase.setMaxOperator(">=");
                        klRuleBase.setMaxValue(importDataVO.getCriticalMax().toString());
                        if (StringUtil.isNotBlank(importDataVO.getUnit())) {
                            klRuleBase.setMaxUnit(importDataVO.getUnit().trim());
                        }
                    }
                    if (StringUtil.isNotBlank(importDataVO.getCriticalNegative())) {
                        klRuleBase.setType(2);
                        klRuleBase.setEqOperator("=");
                        klRuleBase.setEqValue(importDataVO.getCriticalNegative().trim());
                        if (StringUtil.isNotBlank(importDataVO.getUnit())) {
                            klRuleBase.setEqUnit(importDataVO.getUnit().trim());
                        }
                    }

                    klRuleBaseService.save(klRuleBase);
                    KlRuleCondition klRuleCondition = new KlRuleCondition();
                    klRuleCondition.setGroupType(1);
                    klRuleCondition.setRuleId(klRule.getId());
                    klRuleCondition.setRuleBaseId(klRuleBase.getId());
                    klRuleConditionService.save(klRuleCondition);


                    if (StringUtil.isNotBlank(importDataVO.getCriticalAge())) {
                        KlRuleBase klRuleBase2 = new KlRuleBase();
                        klRuleBase2.setConceptId(map.get(importDataVO.getCriticalAge().trim() + "_117").getId());
                        klRuleBase2.setDescription("参考对象:" + importDataVO.getCriticalAge().trim());
                        klRuleBase2.setType(1);
                        klRuleBaseService.save(klRuleBase2);
                        KlRuleCondition klRuleCondition2 = new KlRuleCondition();
                        klRuleCondition2.setGroupType(1);
                        klRuleCondition2.setRuleId(klRule.getId());
                        klRuleCondition2.setRuleBaseId(klRuleBase2.getId());
                        klRuleConditionService.save(klRuleCondition2);
                    }
                }
            }
        }
        log.info("危急值_化验" + lis.size());
    }

    private void pacsCriticalValues(Map<String, String> mapPath, Map<String, KlConcept> map) {
        MultipartFile file = null;
        file = testFacade.getMulFileByPath(mapPath.get("【危急值_辅检】"));
        List<ImportDataVO> data = ExcelUtils.importExcel(file, 0, 1, ImportDataVO.class);
        Integer cnt = 0;
        for (ImportDataVO importDataVO : data) {
            if (null != map.get(importDataVO.getRuleText().trim() + "_112")) {
                KlRule klRule = new KlRule();
                klRule.setConceptId(map.get(importDataVO.getRuleText().trim() + "_112").getId());
                klRule.setDescription("危急值_辅检_" + importDataVO.getRuleText().trim());
                klRule.setRuleType(3);
                klRule.setHasSubCond(0);
                klRuleService.save(klRule);
                cnt++;
            }
        }
        log.info("危急值_辅检" + cnt);
    }

    private void optHighRisk(Map<String, String> mapPath, Map<String, KlConcept> map) {
        MultipartFile file = null;
        file = testFacade.getMulFileByPath(mapPath.get("【高危手术整合】"));
        List<ImportDataVO> data = ExcelUtils.importExcel(file, 0, 1, ImportDataVO.class);
        List<String> opt = new ArrayList<>();
        for (ImportDataVO importDataVO : data) {
            if (StringUtil.isNotBlank(importDataVO.getOperationName())
                    && StringUtil.isNotBlank(importDataVO.getOperationLevel())) {
                if (null != map.get(importDataVO.getOperationName() + "_106").getId()
                        && !opt.contains(importDataVO.getOperationName().trim())) {
                    opt.add(importDataVO.getOperationName().trim());
                    KlRule klRule = new KlRule();
                    klRule.setConceptId(map.get(importDataVO.getOperationName().trim() + "_106").getId());
                    klRule.setDescription("高危手术_" + importDataVO.getOperationName().trim() + importDataVO.getOperationLevel().trim());
                    klRule.setRuleType(2);
                    if (importDataVO.getOperationLevel().trim().equals("2")) {
                        klRule.setHasSubCond(1);
                    } else if (importDataVO.getOperationLevel().trim().equals("3")
                            || importDataVO.getOperationLevel().trim().equals("4")) {
                        klRule.setHasSubCond(0);
                    } else {
                        continue;
                    }
                    klRuleService.save(klRule);
                    if (importDataVO.getOperationLevel().equals("2")) {
                        Integer groupType = 0;
                        if (StringUtil.isNotBlank(importDataVO.getAgeAll())) {
                            groupType = analyDataDeal(map, importDataVO.getAgeAll(), klRule.getId(), 410, groupType);
                        }
                        if (StringUtil.isNotBlank(importDataVO.getPhysique())) {
                            groupType = analyDataDeal(map, importDataVO.getPhysique(), klRule.getId(), 104, 105, groupType);
                        }
                        if (StringUtil.isNotBlank(importDataVO.getLisAll())) {
                            groupType = analyDataDeal(map, importDataVO.getLisAll(), klRule.getId(), 108, groupType);
                        }
                        if (StringUtil.isNotBlank(importDataVO.getDisease())) {
                            groupType = analyDataDealSimple(map, importDataVO.getDisease(), klRule.getId(), 100, groupType);
                        }
                        if (StringUtil.isNotBlank(importDataVO.getPacsAll())) {
                            groupType = analyDataDealSimple(map, importDataVO.getPacsAll(), klRule.getId(), 112, groupType);
                        }
                    }
                }
            }
        }
        log.info("高危手术" + opt.size());
    }

    private void drugHighRisk(Map<String, String> mapPath, Map<String, KlConcept> map) {
        MultipartFile file = null;
        file = testFacade.getMulFileByPath(mapPath.get("【高危药品】"));
        List<ImportDataVO> data = ExcelUtils.importExcel(file, 0, 1, ImportDataVO.class);
        List<String> drugs = new ArrayList<>();
        for (ImportDataVO importDataVO : data) {
            if (StringUtil.isNotBlank(importDataVO.getDrugLevel())
                    && StringUtil.isNotBlank(importDataVO.getDrugHighRisk())
                    && StringUtil.isNotBlank(importDataVO.getDrugForm())) {
                if (null != map.get(importDataVO.getDrugHighRisk() + "_101").getId()
                        && !drugs.contains(importDataVO.getDrugHighRisk().trim())) {
                    drugs.add(importDataVO.getDrugHighRisk().trim());
                    KlRule klRule = new KlRule();
                    klRule.setConceptId(map.get(importDataVO.getDrugHighRisk().trim() + "_101").getId());
                    klRule.setDescription("高危药品_" + importDataVO.getDrugHighRisk().trim());
                    klRule.setRuleType(2);
                    klRule.setMsg(importDataVO.getDrugLevel().trim());
                    klRule.setHasSubCond(1);
                    klRuleService.save(klRule);
                    KlRuleBase klRuleBase = new KlRuleBase();
                    klRuleBase.setType(1);
                    String drugForm = importDataVO.getDrugForm().trim();
                    if (drugForm.indexOf("(") > 0) {
                        drugForm = drugForm.substring(0, importDataVO.getDrugForm().indexOf("("));
                    }
                    klRuleBase.setConceptId(map.get(drugForm + "_102").getId());
                    klRuleBase.setDescription(drugForm);
                    klRuleBaseService.save(klRuleBase);
                    KlRuleCondition klRuleCondition = new KlRuleCondition();
                    klRuleCondition.setGroupType(1);
                    klRuleCondition.setRuleId(klRule.getId());
                    klRuleCondition.setRuleBaseId(klRuleBase.getId());
                    klRuleConditionService.save(klRuleCondition);
                }
            }
        }
        log.info("高危药品" + drugs.size());
    }

    private Integer analyDataDeal(Map<String, KlConcept> map, String str, Long ruleId, Integer type, Integer groupType) {
        return analyDataDeal(map, str, ruleId, type, type, groupType);
    }

    private Integer analyDataDeal(Map<String, KlConcept> map, String str, Long ruleId, Integer condType, Integer noCondType, Integer groupType) {
        KlRuleAnalyVO klRuleAnalyVO = analyData(str);
        return klRuleAnalyVODeal(klRuleAnalyVO, map, ruleId, condType, noCondType, groupType);

    }

    private Integer analyDataDealSimple(Map<String, KlConcept> map, String str, Long ruleId, Integer type, Integer groupType) {
        return analyDataDealSimple(map, str, ruleId, type, type, groupType);
    }

    private Integer analyDataDealSimple(Map<String, KlConcept> map, String str, Long ruleId, Integer condType, Integer noCondType, Integer groupType) {
        KlRuleAnalyVO klRuleAnalyVO = analyDataSimple(str);
        return klRuleAnalyVODeal(klRuleAnalyVO, map, ruleId, condType, noCondType, groupType);

    }

    private Integer klRuleAnalyVODeal(KlRuleAnalyVO klRuleAnalyVO, Map<String, KlConcept> map, Long ruleId, Integer condType, Integer noCondType, Integer groupType) {
        if (ListUtil.isNotEmpty(klRuleAnalyVO.getCondList())) {
            for (KlRuleConditionVO klRuleConditionVO : klRuleAnalyVO.getCondList()) {
                if (null != map.get(klRuleConditionVO.getName() + "_" + condType)) {
                    groupType++;
                    KlRuleBase klRuleBase = new KlRuleBase();
                    klRuleBase.setType(2);
                    klRuleBase.setConceptId(map.get(klRuleConditionVO.getName() + "_" + condType).getId());
                    klRuleBase.setDescription(klRuleConditionVO.getName()
                            + klRuleConditionVO.getOperator()
                            + klRuleConditionVO.getValue()
                            + klRuleConditionVO.getUnit());
                    if (operator_eq.contains(klRuleConditionVO.getOperator())) {
                        klRuleBase.setEqOperator("=");
                        klRuleBase.setEqValue(klRuleConditionVO.getValue());
                        klRuleBase.setEqUnit(klRuleConditionVO.getUnit());
                    } else if (operator_gt.contains(klRuleConditionVO.getOperator())) {
                        klRuleBase.setMaxOperator(">");
                        klRuleBase.setMaxValue(klRuleConditionVO.getValue());
                        klRuleBase.setMaxUnit(klRuleConditionVO.getUnit());
                    } else if (operator_ge.contains(klRuleConditionVO.getOperator())) {
                        klRuleBase.setMaxOperator(">=");
                        klRuleBase.setMaxValue(klRuleConditionVO.getValue());
                        klRuleBase.setMaxUnit(klRuleConditionVO.getUnit());
                    } else if (operator_lt.contains(klRuleConditionVO.getOperator())) {
                        klRuleBase.setMinOperator("<");
                        klRuleBase.setMinValue(klRuleConditionVO.getValue());
                        klRuleBase.setMinUnit(klRuleConditionVO.getUnit());
                    } else if (operator_le.contains(klRuleConditionVO.getOperator())) {
                        klRuleBase.setMinOperator("<=");
                        klRuleBase.setMinValue(klRuleConditionVO.getValue());
                        klRuleBase.setMinUnit(klRuleConditionVO.getUnit());
                    } else {
                        /*DO Nothing*/
                    }
                    klRuleBaseService.save(klRuleBase);
                    KlRuleCondition klRuleCondition = new KlRuleCondition();
                    klRuleCondition.setGroupType(groupType);
                    klRuleCondition.setRuleId(ruleId);
                    klRuleCondition.setRuleBaseId(klRuleBase.getId());
                    klRuleConditionService.save(klRuleCondition);
                }
            }
        }
        if (ListUtil.isNotEmpty(klRuleAnalyVO.getNoCondList())) {
            for (String noCond : klRuleAnalyVO.getNoCondList()) {
                if (null != map.get(noCond + "_" + noCondType)) {
                    groupType++;
                    KlRuleBase klRuleBase = new KlRuleBase();
                    klRuleBase.setType(1);
                    klRuleBase.setConceptId(map.get(noCond + "_" + noCondType).getId());
                    klRuleBase.setDescription(noCond);
                    klRuleBaseService.save(klRuleBase);
                    KlRuleCondition klRuleCondition = new KlRuleCondition();
                    klRuleCondition.setGroupType(groupType);
                    klRuleCondition.setRuleId(ruleId);
                    klRuleCondition.setRuleBaseId(klRuleBase.getId());
                    klRuleConditionService.save(klRuleCondition);
                }
            }
        }
        return groupType;
    }


    private static KlRuleAnalyVO analyDataSimple(String str) {
        KlRuleAnalyVO klRuleAnalyVO = new KlRuleAnalyVO();
        if (StringUtil.isNotBlank(str)) {
            List<String> list = Arrays.asList(str.trim().split("#"));
            klRuleAnalyVO.setNoCondList(list);
        }
        return klRuleAnalyVO;
    }

    private static KlRuleAnalyVO analyData(String str) {
        KlRuleAnalyVO klRuleAnalyVO = new KlRuleAnalyVO();
        if (StringUtil.isNotBlank(str)) {
            List<String> list = Arrays.asList(str.trim().split("，"));
            if (ListUtil.isNotEmpty(list)) {
                List<KlRuleConditionVO> condList = new ArrayList<>();
                List<String> noCondList = new ArrayList<>();
                for (String s : list) {
                    List<String> slist = Arrays.asList(s.trim().split(","));
                    if (slist.size() == 1) {
                        noCondList.add(slist.get(0));
                    } else if (slist.size() == 4) {
                        KlRuleConditionVO klRuleConditionVO = new KlRuleConditionVO();
                        klRuleConditionVO.setName(slist.get(0));
                        klRuleConditionVO.setOperator(slist.get(1));
                        klRuleConditionVO.setValue(slist.get(2));
                        klRuleConditionVO.setUnit(slist.get(3));
                        condList.add(klRuleConditionVO);
                    } else if (slist.size() == 3) {
                        KlRuleConditionVO klRuleConditionVO = new KlRuleConditionVO();
                        klRuleConditionVO.setName(slist.get(0));
                        klRuleConditionVO.setOperator(slist.get(1));
                        klRuleConditionVO.setValue(slist.get(2));
                        klRuleConditionVO.setUnit("");
                        condList.add(klRuleConditionVO);
                    } else if (slist.size() == 2) {
                        KlRuleConditionVO klRuleConditionVO = new KlRuleConditionVO();
                        klRuleConditionVO.setName(slist.get(0));
                        klRuleConditionVO.setOperator("=");
                        klRuleConditionVO.setValue(slist.get(1));
                        klRuleConditionVO.setUnit("");
                        condList.add(klRuleConditionVO);
                    } else {

                    }
                }
                klRuleAnalyVO.setCondList(condList);
                klRuleAnalyVO.setNoCondList(noCondList);
            }
        }
        return klRuleAnalyVO;
    }

    public static void main(String[] args) {
        //analyData("体温,＞,38.0,℃，舒张压,＞,120,mm/Hg，收缩压,＞,180,mm/Hg，舒张压,＜,50,mm/Hg，收缩压,＜,80,mm/Hg，心率,＞,120,次/分，心率,＜,60,次/分，昏迷，意识不清，意识模糊");
        analyData("血小板计数,＜,60,10^9/L，血红蛋白测定(Hb),＜,60,g/L，白细胞计数(WBC),＞,20,10^9/L，国际标准化比值(INR),＞,1.2，血浆凝血酶原时间测定(PT),＞,13,秒，乙型肝炎表面抗原测定(HBsAg)(定量),＞,0.05,IU/ml， 乙型肝炎e抗原测定(HBeAg)(定量),＞,1,S/CO，不加热血清反应素试验,阳性，人免疫缺陷病毒抗体测定(Anti-HIV),阳性，糖化血红蛋白测定,>,8.5,%，血葡萄糖测定,≥,10,mmol/L，钾测定,＞,5.5,mmol/l，钾测定,＜,3.0,mmol/l，血氧饱和度(SAT),<,90,%");
    }
}
